@@ -55,14 +55,7 @@ use polars_arrow::array::ValueSize;
 use std::mem;
 use std::ops::{Deref, DerefMut};
 
-/// Get a 'hash' of the chunks in order to compare chunk sizes quickly.
-fn create_chunk_id(chunks: &[ArrayRef]) -> Vec<usize> {
-    let mut chunk_id = Vec::with_capacity(chunks.len());
-    for a in chunks {
-        chunk_id.push(a.len())
-    }
-    chunk_id
-}
+pub type ChunkIdIter<'a> = std::iter::Map<std::slice::Iter<'a, ArrayRef>, fn(&ArrayRef) -> usize>;
 
 /// # ChunkedArray
 ///
@@ -156,8 +149,6 @@ fn create_chunk_id(chunks: &[ArrayRef]) -> Vec<usize> {
 pub struct ChunkedArray<T> {
     pub(crate) field: Arc<Field>,
     pub(crate) chunks: Vec<ArrayRef>,
-    // chunk lengths
-    chunk_id: Vec<usize>,
     phantom: PhantomData<T>,
     /// maps categorical u32 indexes to String values
     pub(crate) categorical_map: Option<Arc<RevMapping>>,
@@ -182,8 +173,8 @@ impl<T> ChunkedArray<T> {
             Some(0)
         } else {
             let mut offset = 0;
-            for (idx, (null_count, null_bit_buffer)) in self.null_bits().iter().enumerate() {
-                if *null_count == 0 {
+            for (idx, (null_count, null_bit_buffer)) in self.null_bits().enumerate() {
+                if null_count == 0 {
                     return Some(offset);
                 } else {
                     let arr = &self.chunks[idx];
@@ -206,11 +197,8 @@ impl<T> ChunkedArray<T> {
     }
 
     /// Get the null count and the buffer of bits representing null values
-    pub fn null_bits(&self) -> Vec<(usize, Option<Buffer>)> {
-        self.chunks
-            .iter()
-            .map(|arr| get_bitmap(arr.as_ref()))
-            .collect()
+    pub fn null_bits(&self) -> impl Iterator<Item = (usize, Option<Buffer>)> + '_ {
+        self.chunks.iter().map(|arr| get_bitmap(arr.as_ref()))
     }
 
     /// Unpack a Series to the same physical type.
@@ -275,8 +263,8 @@ impl<T> ChunkedArray<T> {
     }
 
     /// Unique id representing the number of chunks
-    pub fn chunk_id(&self) -> &Vec<usize> {
-        &self.chunk_id
+    pub fn chunk_id(&self) -> ChunkIdIter {
+        self.chunks.iter().map(|chunk| chunk.len())
     }
 
     /// A reference to the chunks
@@ -317,7 +305,6 @@ impl<T> ChunkedArray<T> {
         }
         if self.field.data_type() == other.data_type() {
             self.chunks.push(other);
-            self.chunk_id = create_chunk_id(&self.chunks);
             Ok(())
         } else {
             Err(PolarsError::DataTypeMisMatch(
@@ -333,11 +320,9 @@ impl<T> ChunkedArray<T> {
 
     /// Create a new ChunkedArray from self, where the chunks are replaced.
     fn copy_with_chunks(&self, chunks: Vec<ArrayRef>) -> Self {
-        let chunk_id = create_chunk_id(&chunks);
         ChunkedArray {
             field: self.field.clone(),
             chunks,
-            chunk_id,
             phantom: PhantomData,
             categorical_map: self.categorical_map.clone(),
         }
@@ -409,27 +394,6 @@ impl<T> ChunkedArray<T> {
         self.field.data_type()
     }
 
-    /// Get the index of the chunk and the index of the value in that chunk
-    #[inline]
-    pub(crate) fn index_to_chunked_index(&self, index: usize) -> (usize, usize) {
-        if self.chunks.len() == 1 {
-            return (0, index);
-        }
-        let mut index_remainder = index;
-        let mut current_chunk_idx = 0;
-
-        for chunk in &self.chunks {
-            let chunk_len = chunk.len();
-            if chunk_len > index_remainder {
-                break;
-            } else {
-                index_remainder -= chunk_len;
-                current_chunk_idx += 1;
-            }
-        }
-        (current_chunk_idx, index_remainder)
-    }
-
     /// Get the head of the ChunkedArray
     pub fn head(&self, length: Option<usize>) -> Self {
         match length {
@@ -463,9 +427,8 @@ impl<T> ChunkedArray<T> {
         if self.chunks.len() == 1 && self.is_empty() {
             self.chunks = other.chunks.clone();
         } else {
-            self.chunks.extend_from_slice(&other.chunks)
+            self.chunks.extend_from_slice(&other.chunks);
         }
-        self.chunk_id = create_chunk_id(&self.chunks);
     }
 
     /// Name of the ChunkedArray.
@@ -492,18 +455,24 @@ where
     /// Should be used to match the chunk_id of another ChunkedArray.
     /// # Panics
     /// It is the callers responsibility to ensure that this ChunkedArray has a single chunk.
-    pub(crate) fn match_chunks(&self, chunk_id: &[usize]) -> Self {
+    pub(crate) fn match_chunks<I>(&self, chunk_id: I) -> Self
+    where
+        I: Iterator<Item = usize>,
+    {
         debug_assert!(self.chunks.len() == 1);
         // Takes a ChunkedArray containing a single chunk
         let slice = |ca: &Self| {
             let array = &ca.chunks[0];
 
-            let mut chunks = Vec::with_capacity(chunk_id.len());
             let mut offset = 0;
-            for len in chunk_id {
-                chunks.push(array.slice(offset, *len));
-                offset += *len;
-            }
+            let chunks = chunk_id
+                .map(|len| {
+                    let out = array.slice(offset, len);
+                    offset += len;
+                    out
+                })
+                .collect();
+
             Self::new_from_chunks(self.name(), chunks)
         };
 
@@ -533,11 +502,9 @@ where
             T::get_dtype()
         };
         let field = Arc::new(Field::new(name, datatype));
-        let chunk_id = create_chunk_id(&chunks);
         ChunkedArray {
             field,
             chunks,
-            chunk_id,
             phantom: PhantomData,
             categorical_map: None,
         }
@@ -596,7 +563,7 @@ where
                 AnyValue::List(s.unwrap())
             }
             #[cfg(feature = "object")]
-            DataType::Object => AnyValue::Object(&"object"),
+            DataType::Object(_) => AnyValue::Object(&"object"),
             DataType::Categorical => {
                 let v = downcast!(UInt32Array);
                 AnyValue::Utf8(&self.categorical_map.as_ref().expect("should be set").get(v))
@@ -604,28 +571,6 @@ where
             _ => unimplemented!(),
         }
     }
-
-    /// Get a single value. Beware this is slow.
-    /// If you need to use this slightly performant, cast Categorical to UInt32
-    #[inline]
-    pub(crate) unsafe fn get_any_value_unchecked(&self, index: usize) -> AnyValue {
-        let (chunk_idx, idx) = self.index_to_chunked_index(index);
-        debug_assert!(chunk_idx < self.chunks.len());
-        let arr = &**self.chunks.get_unchecked(chunk_idx);
-        debug_assert!(idx < arr.len());
-        self.arr_to_any_value(arr, idx)
-    }
-
-    /// Get a single value. Beware this is slow.
-    /// If you need to use this slightly performant, cast Categorical to UInt32
-    pub(crate) fn get_any_value(&self, index: usize) -> AnyValue {
-        let (chunk_idx, idx) = self.index_to_chunked_index(index);
-        let arr = &*self.chunks[chunk_idx];
-        assert!(idx < arr.len());
-        // SAFETY
-        // bounds are checked
-        unsafe { self.arr_to_any_value(arr, idx) }
-    }
 }
 
 impl<T> ChunkedArray<T>
@@ -644,12 +589,10 @@ where
         values: AlignedVec<T::Native>,
         buffer: Option<Buffer>,
     ) -> Self {
-        let len = values.len();
         let arr = Arc::new(values.into_primitive_array::<T>(buffer));
         ChunkedArray {
             field: Arc::new(Field::new(name, T::get_dtype())),
             chunks: vec![arr],
-            chunk_id: vec![len],
             phantom: PhantomData,
             categorical_map: None,
         }
@@ -818,7 +761,6 @@ impl<T> Clone for ChunkedArray<T> {
         ChunkedArray {
             field: self.field.clone(),
             chunks: self.chunks.clone(),
-            chunk_id: self.chunk_id.clone(),
             phantom: PhantomData,
             categorical_map: self.categorical_map.clone(),
         }
@@ -893,6 +835,7 @@ impl ListChunked {
 #[cfg(test)]
 pub(crate) mod test {
     use crate::prelude::*;
+    use crate::reset_string_cache;
 
     pub(crate) fn get_chunked_array() -> Int32Chunked {
         ChunkedArray::new_from_slice("a", &[1, 2, 3])
@@ -1070,6 +1013,9 @@ pub(crate) mod test {
 
     #[test]
     fn test_iter_categorical() {
+        use crate::SINGLE_LOCK;
+        let _lock = SINGLE_LOCK.lock();
+        reset_string_cache();
         let ca =
             Utf8Chunked::new_from_opt_slice("", &[Some("foo"), None, Some("bar"), Some("ham")]);
         let ca = ca.cast::<CategoricalType>().unwrap();
