@@ -9,6 +9,7 @@ use arrow::array::{ArrayRef, UInt32Array};
 use std::marker::Sized;
 
 pub(crate) mod aggregate;
+pub(crate) mod any_value;
 pub(crate) mod apply;
 pub(crate) mod chunkops;
 pub(crate) mod cum_agg;
@@ -16,17 +17,32 @@ pub(crate) mod downcast;
 pub(crate) mod explode;
 pub(crate) mod fill_none;
 pub(crate) mod filter;
+#[cfg(feature = "is_in")]
+#[cfg_attr(docsrs, doc(cfg(feature = "is_in")))]
 pub(crate) mod is_in;
 pub(crate) mod peaks;
 pub(crate) mod set;
 pub(crate) mod shift;
 pub(crate) mod sort;
 pub(crate) mod take;
+pub(crate) mod take_random;
 pub(crate) mod take_single;
 pub(crate) mod unique;
 pub(crate) mod window;
 pub(crate) mod zip;
 
+pub trait ChunkAnyValue {
+    /// Get a single value. Beware this is slow.
+    /// If you need to use this slightly performant, cast Categorical to UInt32
+    ///
+    /// # Safety
+    /// Does not do any bounds checking.
+    unsafe fn get_any_value_unchecked(&self, index: usize) -> AnyValue;
+
+    /// Get a single value. Beware this is slow.
+    fn get_any_value(&self, index: usize) -> AnyValue;
+}
+
 pub trait ChunkCumAgg<T> {
     /// Get an array with the cumulative max computed at every element
     fn cum_max(&self, _reverse: bool) -> ChunkedArray<T> {
@@ -391,6 +407,8 @@ pub trait ChunkCast {
     fn cast<N>(&self) -> Result<ChunkedArray<N>>
     where
         N: PolarsDataType;
+
+    fn cast_with_dtype(&self, data_type: &DataType) -> Result<Series>;
 }
 
 /// Fastest way to do elementwise operations on a ChunkedArray<T> when the operation is cheaper than
@@ -582,6 +600,13 @@ pub trait ChunkSort<T> {
 
     /// Retrieve the indexes needed to sort this array.
     fn argsort(&self, reverse: bool) -> UInt32Chunked;
+
+    /// Retrieve the indexes need to sort this and the other arrays.
+    fn argsort_multiple(&self, _other: &[Series], _reverse: &[bool]) -> Result<UInt32Chunked> {
+        Err(PolarsError::InvalidOperation(
+            "argsort_multiple not implemented for this dtype".into(),
+        ))
+    }
 }
 
 #[derive(Copy, Clone, Debug)]
@@ -724,6 +749,15 @@ impl ChunkFullNull for ListChunked {
     }
 }
 
+#[cfg(feature = "object")]
+impl<T: PolarsObject> ChunkFullNull for ObjectChunked<T> {
+    fn full_null(name: &str, length: usize) -> ObjectChunked<T> {
+        let mut ca: Self = (0..length).map(|_| None).collect();
+        ca.rename(name);
+        ca
+    }
+}
+
 /// Reverse a ChunkedArray<T>
 pub trait ChunkReverse<T> {
     /// Return a reversed version of this array.
@@ -767,7 +801,7 @@ impl_reverse!(BooleanType, BooleanChunked);
 impl_reverse!(Utf8Type, Utf8Chunked);
 impl_reverse!(ListType, ListChunked);
 #[cfg(feature = "object")]
-impl<T> ChunkReverse<ObjectType<T>> for ObjectChunked<T> {
+impl<T: PolarsObject> ChunkReverse<ObjectType<T>> for ObjectChunked<T> {
     fn reverse(&self) -> Self {
         // Safety
         // we we know we don't get out of bounds
@@ -899,9 +933,11 @@ pub trait ChunkPeaks {
 }
 
 /// Check if element is member of list array
+#[cfg(feature = "is_in")]
+#[cfg_attr(docsrs, doc(cfg(feature = "is_in")))]
 pub trait IsIn {
-    /// Check if the element of this array is in the elements of the list array
-    fn is_in(&self, _list_array: &ListChunked) -> Result<BooleanChunked> {
+    /// Check if elements of this array are in the right Series, or List values of the right Series.
+    fn is_in(&self, _other: &Series) -> Result<BooleanChunked> {
         unimplemented!()
     }
 }
