@@ -3,7 +3,7 @@
 //! At the moment Polars doesn't include all data types available by Arrow. The goal is to
 //! incrementally support more data types and prioritize these by usability.
 //!
-//! [See the AnyType variants](enum.AnyType.html#variants) for the data types that
+//! [See the AnyValue variants](enum.AnyValue.html#variants) for the data types that
 //! are currently supported.
 //!
 use crate::prelude::*;
@@ -17,6 +17,7 @@ pub use arrow::datatypes::{
     TimestampMicrosecondType, TimestampMillisecondType, TimestampNanosecondType,
     TimestampSecondType, UInt16Type, UInt32Type, UInt64Type, UInt8Type,
 };
+use std::cmp::Ordering;
 use std::fmt::{Display, Formatter};
 
 pub struct Utf8Type {}
@@ -98,9 +99,9 @@ pub type ObjectChunked<T> = ChunkedArray<ObjectType<T>>;
 
 #[cfg(feature = "object")]
 #[cfg_attr(docsrs, doc(cfg(feature = "object")))]
-impl<T: Send + Sync> PolarsDataType for ObjectType<T> {
+impl<T: PolarsObject> PolarsDataType for ObjectType<T> {
     fn get_dtype() -> DataType {
-        DataType::Object
+        DataType::Object(T::type_name())
     }
 }
 
@@ -230,6 +231,100 @@ pub enum AnyValue<'a> {
     Object(&'a str),
 }
 
+impl From<f64> for AnyValue<'_> {
+    fn from(a: f64) -> Self {
+        AnyValue::Float64(a)
+    }
+}
+impl From<f32> for AnyValue<'_> {
+    fn from(a: f32) -> Self {
+        AnyValue::Float32(a)
+    }
+}
+impl From<u32> for AnyValue<'_> {
+    fn from(a: u32) -> Self {
+        AnyValue::UInt32(a)
+    }
+}
+impl From<u64> for AnyValue<'_> {
+    fn from(a: u64) -> Self {
+        AnyValue::UInt64(a)
+    }
+}
+impl From<i64> for AnyValue<'_> {
+    fn from(a: i64) -> Self {
+        AnyValue::Int64(a)
+    }
+}
+impl From<i32> for AnyValue<'_> {
+    fn from(a: i32) -> Self {
+        AnyValue::Int32(a)
+    }
+}
+impl From<i16> for AnyValue<'_> {
+    fn from(a: i16) -> Self {
+        AnyValue::Int16(a)
+    }
+}
+impl From<u16> for AnyValue<'_> {
+    fn from(a: u16) -> Self {
+        AnyValue::UInt16(a)
+    }
+}
+
+impl From<i8> for AnyValue<'_> {
+    fn from(a: i8) -> Self {
+        AnyValue::Int8(a)
+    }
+}
+impl From<u8> for AnyValue<'_> {
+    fn from(a: u8) -> Self {
+        AnyValue::UInt8(a)
+    }
+}
+
+impl<'a, T> From<Option<T>> for AnyValue<'a>
+where
+    T: Into<AnyValue<'a>>,
+{
+    fn from(a: Option<T>) -> Self {
+        match a {
+            None => AnyValue::Null,
+            Some(v) => v.into(),
+        }
+    }
+}
+
+impl<'a> AnyValue<'a> {
+    pub fn add<'b>(&self, rhs: &AnyValue<'b>) -> AnyValue<'a> {
+        use AnyValue::*;
+        match (self, rhs) {
+            (Null, _) => Null,
+            (_, Null) => Null,
+            (Int32(l), Int32(r)) => Int32(l + r),
+            (Int64(l), Int64(r)) => Int64(l + r),
+            (UInt32(l), UInt32(r)) => UInt32(l + r),
+            (UInt64(l), UInt64(r)) => UInt64(l + r),
+            (Float32(l), Float32(r)) => Float32(l + r),
+            (Float64(l), Float64(r)) => Float64(l + r),
+            _ => todo!(),
+        }
+    }
+}
+
+impl<'a> From<AnyValue<'a>> for Option<i64> {
+    fn from(val: AnyValue<'a>) -> Self {
+        use AnyValue::*;
+        match val {
+            Null => None,
+            Int32(v) => Some(v as i64),
+            Int64(v) => Some(v as i64),
+            UInt32(v) => Some(v as i64),
+            _ => todo!(),
+        }
+    }
+}
+
 impl Display for DataType {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         let s = match self {
@@ -253,7 +348,7 @@ impl Display for DataType {
             DataType::Duration(TimeUnit::Millisecond) => "duration(ms)",
             DataType::List(tp) => return write!(f, "list [{}]", DataType::from(tp)),
             #[cfg(feature = "object")]
-            DataType::Object => "object",
+            DataType::Object(s) => s,
             DataType::Categorical => "cat",
             _ => panic!("{:?} not implemented", self),
         };
@@ -261,7 +356,7 @@ impl Display for DataType {
     }
 }
 
-impl<'a> PartialEq for AnyValue<'a> {
+impl PartialEq for AnyValue<'_> {
     // Everything of Any is slow. Don't use.
     fn eq(&self, other: &Self) -> bool {
         use AnyValue::*;
@@ -292,6 +387,26 @@ impl<'a> PartialEq for AnyValue<'a> {
     }
 }
 
+impl PartialOrd for AnyValue<'_> {
+    /// Only implemented for the same types and physical types!
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        use AnyValue::*;
+        match (self, other) {
+            (UInt8(l), UInt8(r)) => l.partial_cmp(r),
+            (UInt16(l), UInt16(r)) => l.partial_cmp(r),
+            (UInt32(l), UInt32(r)) => l.partial_cmp(r),
+            (UInt64(l), UInt64(r)) => l.partial_cmp(r),
+            (Int8(l), Int8(r)) => l.partial_cmp(r),
+            (Int16(l), Int16(r)) => l.partial_cmp(r),
+            (Int32(l), Int32(r)) => l.partial_cmp(r),
+            (Int64(l), Int64(r)) => l.partial_cmp(r),
+            (Float32(l), Float32(r)) => l.partial_cmp(r),
+            (Float64(l), Float64(r)) => l.partial_cmp(r),
+            _ => None,
+        }
+    }
+}
+
 #[derive(Clone, PartialEq, Eq, Debug, Hash)]
 pub enum DataType {
     Boolean,
@@ -316,7 +431,9 @@ pub enum DataType {
     List(ArrowDataType),
     Duration(TimeUnit),
     #[cfg(feature = "object")]
-    Object,
+    /// A generic type that can be used in a `Series`
+    /// &'static str can be used to determine/set inner type
+    Object(&'static str),
     Null,
     Categorical,
 }
@@ -346,8 +463,8 @@ impl DataType {
             Duration(tu) => ArrowDataType::Duration(tu.clone()),
             Null => ArrowDataType::Null,
             #[cfg(feature = "object")]
-            Object => unimplemented!(),
-            Categorical => ArrowDataType::UInt16,
+            Object(_) => unimplemented!(),
+            Categorical => ArrowDataType::UInt32,
         }
     }
 }
