@@ -21,6 +21,7 @@ pub(crate) mod pivot;
 pub mod resample;
 
 pub type GroupTuples = Vec<(u32, Vec<u32>)>;
+pub type GroupedMap<T> = HashMap<T, Vec<u32>, RandomState>;
 
 fn groupby<T>(a: impl Iterator<Item = T>) -> GroupTuples
 where
@@ -574,22 +575,18 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     }
 
     pub fn keys(&self) -> Vec<Series> {
-        // Keys will later be appended with the aggregation columns, so we already allocate extra space
-        let size;
-        if let Some(sel) = &self.selected_agg {
-            size = sel.len() + self.selected_keys.len();
-        } else {
-            size = self.selected_keys.len();
-        }
-        let mut keys = Vec::with_capacity(size);
-        unsafe {
-            self.selected_keys.iter().for_each(|s| {
-                let key =
-                    s.take_iter_unchecked(&mut self.groups.iter().map(|(idx, _)| *idx as usize));
-                keys.push(key)
-            });
-        }
-        keys
+        POOL.install(|| {
+            self.selected_keys
+                .par_iter()
+                .map(|s| {
+                    // Safety
+                    // groupby indexes are in bound.
+                    unsafe {
+                        s.take_iter_unchecked(&mut self.groups.iter().map(|(idx, _)| *idx as usize))
+                    }
+                })
+                .collect()
+        })
     }
 
     fn prepare_agg(&self) -> Result<(Vec<Series>, Vec<Series>)> {
@@ -1259,6 +1256,7 @@ mod test {
     use crate::frame::groupby::{groupby, groupby_threaded_flat};
     use crate::prelude::*;
     use crate::utils::split_ca;
+    use num::traits::FloatConst;
 
     #[test]
     #[cfg(feature = "dtype-date32")]
@@ -1525,4 +1523,45 @@ mod test {
             assert_eq!(a, b);
         }
     }
+
+    #[test]
+    fn test_groupby_null_handling() -> Result<()> {
+        let df = df!(
+            "a" => ["a", "a", "a", "b", "b"],
+            "b" => [Some(1), Some(2), None, None, Some(1)]
+        )?;
+        let out = df.groupby_stable("a")?.mean()?;
+
+        assert_eq!(
+            Vec::from(out.column("b_mean")?.f64()?),
+            &[Some(1.5), Some(1.0)]
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn test_groupby_var() -> Result<()> {
+        // check variance and proper coercion to f64
+        let df = df![
+            "g" => ["foo", "foo", "bar"],
+            "flt" => [1.0, 2.0, 3.0],
+            "int" => [1, 2, 3]
+        ]?;
+
+        let out = df.groupby("g")?.select("int").var()?;
+        assert_eq!(
+            out.column("int_agg_var")?.f64()?.sort(false).get(1),
+            Some(0.5)
+        );
+        let out = df.groupby("g")?.select("int").std()?;
+        let val = out
+            .column("int_agg_std")?
+            .f64()?
+            .sort(false)
+            .get(1)
+            .unwrap();
+        let expected = f64::FRAC_1_SQRT_2();
+        assert!((val - expected).abs() < 0.000001);
+        Ok(())
+    }
 }
