@@ -13,7 +13,10 @@ use rayon::prelude::*;
 use crate::chunked_array::ops::unique::is_unique_helper;
 use crate::frame::select::Selection;
 use crate::prelude::*;
-use crate::utils::{accumulate_dataframes_horizontal, accumulate_dataframes_vertical, NoNull};
+use crate::utils::{
+    accumulate_dataframes_horizontal, accumulate_dataframes_vertical, get_supertype, split_ca,
+    split_df, NoNull,
+};
 
 mod arithmetic;
 pub mod explode;
@@ -22,6 +25,7 @@ pub mod hash_join;
 pub mod row;
 pub mod select;
 mod upstream_traits;
+use crate::POOL;
 
 #[derive(Clone)]
 pub struct DataFrame {
@@ -107,28 +111,43 @@ impl DataFrame {
         Ok(df)
     }
 
-    // doesn't check Series sizes.
-    // todo! make private
+    /// Create a new `DataFrame` but does not check the length or duplicate occurrence of the `Series`.
+    ///
+    /// It is adviced to use [Series::new](Series::new) in favor of this method.
+    ///
+    /// # Panic
+    /// It is the callers responsibility to uphold the contract of all `Series`
+    /// having an equal length, if not this may panic down the line.
     pub fn new_no_checks(columns: Vec<Series>) -> DataFrame {
         DataFrame { columns }
     }
 
     /// Aggregate all chunks to contiguous memory.
     pub fn agg_chunks(&self) -> Self {
+        // Don't parallelize this. Memory overhead
         let f = |s: &Series| s.rechunk();
-        let cols = self.columns.par_iter().map(f).collect();
+        let cols = self.columns.iter().map(f).collect();
         DataFrame::new_no_checks(cols)
     }
 
     /// Aggregate all the chunks in the DataFrame to a single chunk.
     pub fn as_single_chunk(&mut self) -> &mut Self {
-        self.columns = self.columns.iter().map(|s| s.rechunk()).collect();
+        // Don't parallelize this. Memory overhead
+        for s in &mut self.columns {
+            *s = s.rechunk();
+        }
         self
     }
 
     /// Ensure all the chunks in the DataFrame are aligned.
     pub fn rechunk(&mut self) -> &mut Self {
-        if self.columns.iter().map(|s| s.chunk_lengths()).all_equal() {
+        // TODO: remove vec allocation
+        if self
+            .columns
+            .iter()
+            .map(|s| s.chunk_lengths().collect_vec())
+            .all_equal()
+        {
             self
         } else {
             self.as_single_chunk()
@@ -278,7 +297,7 @@ impl DataFrame {
         // first loop check validity. We don't do this in a single pass otherwise
         // this DataFrame is already modified when an error occurs.
         for col in columns {
-            if col.len() != height {
+            if col.len() != height && height != 0 {
                 return Err(PolarsError::ShapeMisMatch(
                     format!("Could not horizontally stack Series. The Series length {} differs from the DataFrame height: {}", col.len(), height).into()));
             }
@@ -321,22 +340,23 @@ impl DataFrame {
             ));
         }
 
-        if self.dtypes() != df.dtypes() {
-            return Err(PolarsError::DataTypeMisMatch(
-                format!(
-                    "cannot vstack: data types don't match of {:?} {:?}",
-                    self.head(Some(2)),
-                    df.head(Some(2))
-                )
-                .into(),
-            ));
-        }
         self.columns
             .iter_mut()
             .zip(df.columns.iter())
-            .for_each(|(left, right)| {
+            .try_for_each(|(left, right)| {
+                if left.dtype() != right.dtype() {
+                    return Err(PolarsError::DataTypeMisMatch(
+                        format!(
+                            "cannot vstack: data types don't match of {:?} {:?}",
+                            left, right
+                        )
+                        .into(),
+                    ));
+                }
+
                 left.append(right).expect("should not fail");
-            });
+                Ok(())
+            })?;
         // don't rechunk here. Chunks in columns always match.
         Ok(self)
     }
@@ -560,13 +580,62 @@ impl DataFrame {
         }
     }
 
+    /// Does a filter but splits thread chunks vertically instead of horizontally
+    /// This yields a DataFrame with `n_chunks == n_threads`.
+    fn filter_vertical(&self, mask: &BooleanChunked) -> Result<Self> {
+        let n_threads = POOL.current_num_threads();
+
+        let masks = split_ca(mask, n_threads).unwrap();
+        let dfs = split_df(self, n_threads).unwrap();
+        let dfs: Result<Vec<_>> = POOL.install(|| {
+            masks
+                .par_iter()
+                .zip(dfs)
+                .map(|(mask, df)| {
+                    let cols = df
+                        .columns
+                        .iter()
+                        .map(|s| s.filter(mask))
+                        .collect::<Result<_>>()?;
+                    Ok(DataFrame::new_no_checks(cols))
+                })
+                .collect()
+        });
+
+        let mut iter = dfs?.into_iter();
+        let first = iter.next().unwrap();
+        Ok(iter.fold(first, |mut acc, df| {
+            acc.vstack_mut(&df).unwrap();
+            acc
+        }))
+    }
+
     /// Take DataFrame rows by a boolean mask.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use polars_core::prelude::*;
+    /// fn example(df: &DataFrame) -> Result<DataFrame> {
+    ///     let mask = df.column("sepal.width")?.is_not_null();
+    ///     df.filter(&mask)
+    /// }
+    ///
+    /// ```
     pub fn filter(&self, mask: &BooleanChunked) -> Result<Self> {
-        let new_col = self
-            .columns
-            .par_iter()
-            .map(|col| col.filter(mask))
-            .collect::<Result<Vec<_>>>()?;
+        if std::env::var("POLARS_VERT_PAR").is_ok() {
+            return self.filter_vertical(mask);
+        }
+
+        let new_col = POOL.install(|| {
+            self.columns
+                .par_iter()
+                .map(|s| match s.dtype() {
+                    DataType::Utf8 => s.filter_threaded(mask, true),
+                    _ => s.filter(mask),
+                })
+                .collect::<Result<Vec<_>>>()
+        })?;
         Ok(DataFrame::new_no_checks(new_col))
     }
 
@@ -609,23 +678,24 @@ impl DataFrame {
     where
         I: Iterator<Item = usize> + Clone + Sync,
     {
+        if std::env::var("POLARS_VERT_PAR").is_ok() {
+            let idx_ca: NoNull<UInt32Chunked> = iter.into_iter().map(|idx| idx as u32).collect();
+            return self.take_unchecked_vectical(&idx_ca.into_inner());
+        }
+
         let n_chunks = match self.n_chunks() {
             Err(_) => return self.clone(),
             Ok(n) => n,
         };
+        let has_utf8 = self
+            .columns
+            .iter()
+            .any(|s| matches!(s.dtype(), DataType::Utf8));
 
-        if n_chunks == 1 {
+        if n_chunks == 1 || has_utf8 {
             let idx_ca: NoNull<UInt32Chunked> = iter.into_iter().map(|idx| idx as u32).collect();
             let idx_ca = idx_ca.into_inner();
-            let cols = self
-                .columns
-                .par_iter()
-                .map(|s| {
-                    s.take_unchecked(&idx_ca)
-                        .expect("already checked single chunk")
-                })
-                .collect();
-            return DataFrame::new_no_checks(cols);
+            return self.take_unchecked(&idx_ca);
         }
 
         let new_col = self
@@ -653,22 +723,24 @@ impl DataFrame {
     where
         I: Iterator<Item = Option<usize>> + Clone + Sync,
     {
+        if std::env::var("POLARS_VERT_PAR").is_ok() {
+            let idx_ca: UInt32Chunked = iter.into_iter().map(|opt| opt.map(|v| v as u32)).collect();
+            return self.take_unchecked_vectical(&idx_ca);
+        }
+
         let n_chunks = match self.n_chunks() {
             Err(_) => return self.clone(),
             Ok(n) => n,
         };
 
-        if n_chunks == 1 {
+        let has_utf8 = self
+            .columns
+            .iter()
+            .any(|s| matches!(s.dtype(), DataType::Utf8));
+
+        if n_chunks == 1 || has_utf8 {
             let idx_ca: UInt32Chunked = iter.into_iter().map(|opt| opt.map(|v| v as u32)).collect();
-            let cols = self
-                .columns
-                .par_iter()
-                .map(|s| {
-                    s.take_unchecked(&idx_ca)
-                        .expect("already checked single chunk")
-                })
-                .collect();
-            return DataFrame::new_no_checks(cols);
+            return self.take_unchecked(&idx_ca);
         }
 
         let new_col = self
@@ -706,11 +778,57 @@ impl DataFrame {
         } else {
             Cow::Borrowed(indices)
         };
-        let new_col = self.columns.par_iter().map(|s| s.take(&indices)).collect();
+        let new_col = POOL.install(|| {
+            self.columns
+                .par_iter()
+                .map(|s| match s.dtype() {
+                    DataType::Utf8 => s.take_threaded(&indices, true),
+                    _ => s.take(&indices),
+                })
+                .collect()
+        });
 
         DataFrame::new_no_checks(new_col)
     }
 
+    unsafe fn take_unchecked(&self, idx: &UInt32Chunked) -> Self {
+        let cols = POOL.install(|| {
+            self.columns
+                .par_iter()
+                .map(|s| match s.dtype() {
+                    DataType::Utf8 => s.take_unchecked_threaded(&idx, true).unwrap(),
+                    _ => s.take_unchecked(&idx).unwrap(),
+                })
+                .collect()
+        });
+        DataFrame::new_no_checks(cols)
+    }
+
+    unsafe fn take_unchecked_vectical(&self, indices: &UInt32Chunked) -> Self {
+        let n_threads = POOL.current_num_threads();
+        let idxs = split_ca(&indices, n_threads).unwrap();
+
+        let dfs: Vec<_> = POOL.install(|| {
+            idxs.par_iter()
+                .map(|idx| {
+                    let cols = self
+                        .columns
+                        .iter()
+                        .map(|s| s.take_unchecked(idx).unwrap())
+                        .collect();
+                    DataFrame::new_no_checks(cols)
+                })
+                .collect()
+        });
+
+        let mut iter = dfs.into_iter();
+        let first = iter.next().unwrap();
+        iter.fold(first, |mut acc, df| {
+            acc.vstack_mut(&df).unwrap();
+            acc
+        })
+    }
+
     /// Rename a column in the DataFrame
     ///
     /// # Example
@@ -732,22 +850,98 @@ impl DataFrame {
 
     /// Sort DataFrame in place by a column.
     pub fn sort_in_place(&mut self, by_column: &str, reverse: bool) -> Result<&mut Self> {
-        let s = self.column(by_column)?;
-
-        let take = s.argsort(reverse);
-
-        self.columns = self.columns.par_iter().map(|s| s.take(&take)).collect();
+        self.columns = self.sort(by_column, reverse)?.columns;
         Ok(self)
     }
 
-    /// Return a sorted clone of this DataFrame.
-    pub fn sort(&self, by_column: &str, reverse: bool) -> Result<Self> {
-        let s = self.column(by_column)?;
+    /// This is the dispatch of Self::sort, and exists to reduce compile bloat by monomorphization.
+    fn sort_impl(&self, by_column: Vec<&str>, mut reverse: Vec<bool>) -> Result<Self> {
+        let take = match by_column.len() {
+            1 => {
+                let s = self.column(by_column[0])?;
+                s.argsort(reverse[0])
+            }
+            n_cols => {
+                #[cfg(feature = "sort_multiple")]
+                {
+                    let mut columns = self.select_series(by_column)?;
 
-        let take = s.argsort(reverse);
+                    // we only allow this implementation of the same types
+                    // se we determine the supertypes and coerce all series.
+                    let mut first = columns.remove(0);
+                    let dtype = if first.utf8().is_ok() {
+                        Some(DataType::Float64)
+                    } else {
+                        columns.iter().try_fold::<_, _, Result<_>>(None, |acc, s| {
+                            let acc = match (&acc, s.dtype()) {
+                                (_, DataType::Utf8) => acc,
+                                (None, dt) => Some(dt.clone()),
+                                (Some(acc), dt) => Some(get_supertype(acc, dt)?),
+                            };
+                            Ok(acc)
+                        })?
+                    };
+
+                    if let Some(dtype) = dtype {
+                        columns = columns
+                            .into_iter()
+                            .map(|s| match s.dtype() {
+                                DataType::Utf8 => s,
+                                _ => s.cast_with_dtype(&dtype).expect("supertype is known"),
+                            })
+                            .collect::<Vec<_>>();
+
+                        // broadcast ordering
+                        if n_cols > reverse.len() && reverse.len() == 1 {
+                            while n_cols != reverse.len() {
+                                reverse.push(reverse[0]);
+                            }
+                        }
+
+                        if !matches!(first.dtype(), DataType::Utf8) {
+                            first = first.cast_with_dtype(&dtype)?;
+                        }
+                    }
+
+                    first.argsort_multiple(&columns, &reverse)?
+                }
+                #[cfg(not(feature = "sort_multiple"))]
+                {
+                    panic!("activate `sort_multiple` feature gate to enable this functionality");
+                }
+            }
+        };
+        if std::env::var("POLARS_VERT_PAR").is_ok() {
+            return Ok(unsafe { self.take_unchecked_vectical(&take) });
+        }
         Ok(self.take(&take))
     }
 
+    /// Return a sorted clone of this DataFrame.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use polars_core::prelude::*;
+    ///
+    /// fn sort_example(df: &DataFrame, reverse: bool) -> Result<DataFrame> {
+    ///     df.sort("a", reverse)
+    /// }
+    ///
+    /// fn sort_by_multiple_columns_example(df: &DataFrame) -> Result<DataFrame> {
+    ///     df.sort(&["a", "b"], vec![false, true])
+    /// }
+    /// ```
+    pub fn sort<'a, S, J>(&self, by_column: S, reverse: impl IntoVec<bool>) -> Result<Self>
+    where
+        S: Selection<'a, J>,
+    {
+        // we do this heap allocation and dispatch to reduce monomorphization bloat
+        let by_column = by_column.to_selection_vec();
+        let reverse = reverse.into_vec();
+        self.sort_impl(by_column, reverse)
+    }
+
     /// Replace a column with a series.
     pub fn replace<S: IntoSeries>(&mut self, column: &str, new_col: S) -> Result<&mut Self> {
         self.apply(column, |_| new_col.into_series())
