@@ -10,8 +10,7 @@ use super::private;
 use super::IntoSeries;
 use super::SeriesTrait;
 use super::SeriesWrap;
-use crate::chunked_array::comparison::*;
-use crate::chunked_array::AsSinglePtr;
+use crate::chunked_array::{comparison::*, AsSinglePtr, ChunkIdIter};
 use crate::fmt::FmtList;
 #[cfg(feature = "pivot")]
 use crate::frame::groupby::pivot::*;
@@ -37,12 +36,12 @@ macro_rules! physical_dispatch {
     ($s: expr, $method: ident, $($args:expr),*) => {{
         let dtype = $s.dtype();
         let phys_type = $s.physical_type();
-        let s = $s.cast_with_datatype(&phys_type).unwrap();
+        let s = $s.cast_with_dtype(&phys_type).unwrap();
         let s = s.$method($($args),*);
 
         // if the type is unchanged we return the original type
         if s.dtype() == &phys_type {
-            s.cast_with_datatype(dtype).unwrap()
+            s.cast_with_dtype(dtype).unwrap()
         }
         // else the change of type is part of the operation.
         else {
@@ -55,12 +54,12 @@ macro_rules! try_physical_dispatch {
     ($s: expr, $method: ident, $($args:expr),*) => {{
         let dtype = $s.dtype();
         let phys_type = $s.physical_type();
-        let s = $s.cast_with_datatype(&phys_type).unwrap();
+        let s = $s.cast_with_dtype(&phys_type).unwrap();
         let s = s.$method($($args),*)?;
 
         // if the type is unchanged we return the original type
         if s.dtype() == &phys_type {
-            s.cast_with_datatype(dtype)
+            s.cast_with_dtype(dtype)
         }
         // else the change of type is part of the operation.
         else {
@@ -73,12 +72,12 @@ macro_rules! opt_physical_dispatch {
     ($s: expr, $method: ident, $($args:expr),*) => {{
         let dtype = $s.dtype();
         let phys_type = $s.physical_type();
-        let s = $s.cast_with_datatype(&phys_type).unwrap();
+        let s = $s.cast_with_dtype(&phys_type).unwrap();
         let s = s.$method($($args),*)?;
 
         // if the type is unchanged we return the original type
         if s.dtype() == &phys_type {
-            Some(s.cast_with_datatype(dtype).unwrap())
+            Some(s.cast_with_dtype(dtype).unwrap())
         }
         // else the change of type is part of the operation.
         else {
@@ -91,7 +90,7 @@ macro_rules! opt_physical_dispatch {
 macro_rules! cast_and_apply {
     ($s: expr, $method: ident, $($args:expr),*) => {{
         let phys_type = $s.physical_type();
-        let s = $s.cast_with_datatype(&phys_type).unwrap();
+        let s = $s.cast_with_dtype(&phys_type).unwrap();
         s.$method($($args),*)
     }}
 }
@@ -118,10 +117,6 @@ macro_rules! impl_dyn_series {
                 try_physical_dispatch!(self, zip_with_same_type, mask, other)
             }
 
-            fn is_in_same_type(&self, list_array: &ListChunked) -> Result<BooleanChunked> {
-                cast_and_apply!(self, is_in_same_type, list_array)
-            }
-
             fn vec_hash(&self, random_state: RandomState) -> UInt64Chunked {
                 cast_and_apply!(self, vec_hash, random_state)
             }
@@ -174,6 +169,10 @@ macro_rules! impl_dyn_series {
             fn agg_median(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
                 opt_physical_dispatch!(self, agg_median, groups)
             }
+            #[cfg(feature = "lazy")]
+            fn agg_valid_count(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+                opt_physical_dispatch!(self, agg_valid_count, groups)
+            }
 
             #[cfg(feature = "pivot")]
             fn pivot<'a>(
@@ -233,6 +232,15 @@ macro_rules! impl_dyn_series {
             fn group_tuples(&self, multithreaded: bool) -> GroupTuples {
                 cast_and_apply!(self, group_tuples, multithreaded)
             }
+            #[cfg(feature = "sort_multiple")]
+            fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<UInt32Chunked> {
+                let phys_type = self.0.physical_type();
+                let s = self.cast_with_dtype(&phys_type).unwrap();
+
+                self.0
+                    .unpack_series_matching_type(&s)?
+                    .argsort_multiple(by, reverse)
+            }
         }
 
         impl SeriesTrait for SeriesWrap<$ca> {
@@ -256,7 +264,7 @@ macro_rules! impl_dyn_series {
                 self.0.array_data()
             }
 
-            fn chunk_lengths(&self) -> &Vec<usize> {
+            fn chunk_lengths(&self) -> ChunkIdIter {
                 self.0.chunk_id()
             }
             fn name(&self) -> &str {
@@ -435,52 +443,8 @@ macro_rules! impl_dyn_series {
                 physical_dispatch!(self, expand_at_index, index, length)
             }
 
-            fn cast_with_datatype(&self, data_type: &DataType) -> Result<Series> {
-                use DataType::*;
-                match data_type {
-                    Boolean => ChunkCast::cast::<BooleanType>(&self.0).map(|ca| ca.into_series()),
-                    Utf8 => ChunkCast::cast::<Utf8Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-u8")]
-                    UInt8 => ChunkCast::cast::<UInt8Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-u16")]
-                    UInt16 => ChunkCast::cast::<UInt16Type>(&self.0).map(|ca| ca.into_series()),
-                    UInt32 => ChunkCast::cast::<UInt32Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-u64")]
-                    UInt64 => ChunkCast::cast::<UInt64Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-i8")]
-                    Int8 => ChunkCast::cast::<Int8Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-i16")]
-                    Int16 => ChunkCast::cast::<Int16Type>(&self.0).map(|ca| ca.into_series()),
-                    Int32 => ChunkCast::cast::<Int32Type>(&self.0).map(|ca| ca.into_series()),
-                    Int64 => ChunkCast::cast::<Int64Type>(&self.0).map(|ca| ca.into_series()),
-                    Float32 => ChunkCast::cast::<Float32Type>(&self.0).map(|ca| ca.into_series()),
-                    Float64 => ChunkCast::cast::<Float64Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-date32")]
-                    Date32 => ChunkCast::cast::<Date32Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-date64")]
-                    Date64 => ChunkCast::cast::<Date64Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-time64-ns")]
-                    Time64(TimeUnit::Nanosecond) => {
-                        ChunkCast::cast::<Time64NanosecondType>(&self.0).map(|ca| ca.into_series())
-                    }
-                    #[cfg(feature = "dtype-duration-ns")]
-                    Duration(TimeUnit::Nanosecond) => {
-                        ChunkCast::cast::<DurationNanosecondType>(&self.0)
-                            .map(|ca| ca.into_series())
-                    }
-                    #[cfg(feature = "dtype-duration-ms")]
-                    Duration(TimeUnit::Millisecond) => {
-                        ChunkCast::cast::<DurationMillisecondType>(&self.0)
-                            .map(|ca| ca.into_series())
-                    }
-                    List(_) => ChunkCast::cast::<ListType>(&self.0).map(|ca| ca.into_series()),
-                    Categorical => {
-                        ChunkCast::cast::<CategoricalType>(&self.0).map(|ca| ca.into_series())
-                    }
-                    dt => Err(PolarsError::Other(
-                        format!("Casting to {:?} is not supported", dt).into(),
-                    )),
-                }
+            fn cast_with_dtype(&self, data_type: &DataType) -> Result<Series> {
+                self.0.cast_with_dtype(data_type)
             }
 
             fn to_dummies(&self) -> Result<DataFrame> {
@@ -558,7 +522,7 @@ macro_rules! impl_dyn_series {
             }
 
             fn null_bits(&self) -> Vec<(usize, Option<Buffer>)> {
-                self.0.null_bits()
+                self.0.null_bits().collect()
             }
 
             fn reverse(&self) -> Series {
@@ -697,6 +661,10 @@ macro_rules! impl_dyn_series {
             fn peak_min(&self) -> BooleanChunked {
                 cast_and_apply!(self, peak_min,)
             }
+            #[cfg(feature = "is_in")]
+            fn is_in(&self, other: &Series) -> Result<BooleanChunked> {
+                IsIn::is_in(&self.0, other)
+            }
         }
     };
 }
@@ -720,7 +688,7 @@ mod test {
     #[cfg(feature = "dtype-date64")]
     fn test_agg_list_type() -> Result<()> {
         let s = Series::new("foo", &[1, 2, 3]);
-        let s = s.cast_with_datatype(&DataType::Date64)?;
+        let s = s.cast_with_dtype(&DataType::Date64)?;
 
         let l = s.agg_list(&[(0, vec![0, 1, 2])]).unwrap();
         assert!(matches!(l.dtype(), DataType::List(ArrowDataType::Date64)));
@@ -732,7 +700,7 @@ mod test {
     #[cfg(feature = "dtype-date64")]
     fn test_datelike_join() -> Result<()> {
         let s = Series::new("foo", &[1, 2, 3]);
-        let mut s1 = s.cast_with_datatype(&DataType::Date64)?;
+        let mut s1 = s.cast_with_dtype(&DataType::Date64)?;
         s1.rename("bar");
 
         let df = DataFrame::new(vec![s, s1])?;
@@ -752,7 +720,7 @@ mod test {
     #[cfg(feature = "dtype-date64")]
     fn test_datelike_methods() -> Result<()> {
         let s = Series::new("foo", &[1, 2, 3]);
-        let s = s.cast_with_datatype(&DataType::Date64)?;
+        let s = s.cast_with_dtype(&DataType::Date64)?;
 
         let out = s.subtract(&s)?;
         assert!(matches!(out.dtype(), DataType::Date64));
