@@ -15,7 +15,7 @@ use super::SeriesTrait;
 use crate::chunked_array::comparison::*;
 use crate::chunked_array::{
     ops::aggregate::{ChunkAggSeries, VarAggSeries},
-    AsSinglePtr,
+    AsSinglePtr, ChunkIdIter,
 };
 use crate::fmt::FmtList;
 #[cfg(feature = "pivot")]
@@ -69,10 +69,6 @@ macro_rules! impl_dyn_series {
                     .map(|ca| ca.into_series())
             }
 
-            fn is_in_same_type(&self, list_array: &ListChunked) -> Result<BooleanChunked> {
-                IsIn::is_in(&self.0, list_array)
-            }
-
             fn vec_hash(&self, random_state: RandomState) -> UInt64Chunked {
                 self.0.vec_hash(random_state)
             }
@@ -124,6 +120,10 @@ macro_rules! impl_dyn_series {
             fn agg_median(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
                 self.0.agg_median(groups)
             }
+            #[cfg(feature = "lazy")]
+            fn agg_valid_count(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+                self.0.agg_valid_count(groups)
+            }
 
             #[cfg(feature = "pivot")]
             fn pivot<'a>(
@@ -179,6 +179,11 @@ macro_rules! impl_dyn_series {
             fn group_tuples(&self, multithreaded: bool) -> GroupTuples {
                 IntoGroupTuples::group_tuples(&self.0, multithreaded)
             }
+
+            #[cfg(feature = "sort_multiple")]
+            fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<UInt32Chunked> {
+                self.0.argsort_multiple(by, reverse)
+            }
         }
 
         impl SeriesTrait for SeriesWrap<$ca> {
@@ -202,7 +207,7 @@ macro_rules! impl_dyn_series {
                 self.0.array_data()
             }
 
-            fn chunk_lengths(&self) -> &Vec<usize> {
+            fn chunk_lengths(&self) -> ChunkIdIter {
                 self.0.chunk_id()
             }
             fn name(&self) -> &str {
@@ -602,52 +607,8 @@ macro_rules! impl_dyn_series {
                 ChunkExpandAtIndex::expand_at_index(&self.0, index, length).into_series()
             }
 
-            fn cast_with_datatype(&self, data_type: &DataType) -> Result<Series> {
-                use DataType::*;
-                match data_type {
-                    Boolean => ChunkCast::cast::<BooleanType>(&self.0).map(|ca| ca.into_series()),
-                    Utf8 => ChunkCast::cast::<Utf8Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-u8")]
-                    UInt8 => ChunkCast::cast::<UInt8Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-u16")]
-                    UInt16 => ChunkCast::cast::<UInt16Type>(&self.0).map(|ca| ca.into_series()),
-                    UInt32 => ChunkCast::cast::<UInt32Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-u64")]
-                    UInt64 => ChunkCast::cast::<UInt64Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-i8")]
-                    Int8 => ChunkCast::cast::<Int8Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-i16")]
-                    Int16 => ChunkCast::cast::<Int16Type>(&self.0).map(|ca| ca.into_series()),
-                    Int32 => ChunkCast::cast::<Int32Type>(&self.0).map(|ca| ca.into_series()),
-                    Int64 => ChunkCast::cast::<Int64Type>(&self.0).map(|ca| ca.into_series()),
-                    Float32 => ChunkCast::cast::<Float32Type>(&self.0).map(|ca| ca.into_series()),
-                    Float64 => ChunkCast::cast::<Float64Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-date32")]
-                    Date32 => ChunkCast::cast::<Date32Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-date64")]
-                    Date64 => ChunkCast::cast::<Date64Type>(&self.0).map(|ca| ca.into_series()),
-                    #[cfg(feature = "dtype-time64-ns")]
-                    Time64(TimeUnit::Nanosecond) => {
-                        ChunkCast::cast::<Time64NanosecondType>(&self.0).map(|ca| ca.into_series())
-                    }
-                    #[cfg(feature = "dtype-duration-ns")]
-                    Duration(TimeUnit::Nanosecond) => {
-                        ChunkCast::cast::<DurationNanosecondType>(&self.0)
-                            .map(|ca| ca.into_series())
-                    }
-                    #[cfg(feature = "dtype-duration-ms")]
-                    Duration(TimeUnit::Millisecond) => {
-                        ChunkCast::cast::<DurationMillisecondType>(&self.0)
-                            .map(|ca| ca.into_series())
-                    }
-                    List(_) => ChunkCast::cast::<ListType>(&self.0).map(|ca| ca.into_series()),
-                    Categorical => {
-                        ChunkCast::cast::<CategoricalType>(&self.0).map(|ca| ca.into_series())
-                    }
-                    dt => Err(PolarsError::Other(
-                        format!("Casting to {:?} is not supported", dt).into(),
-                    )),
-                }
+            fn cast_with_dtype(&self, data_type: &DataType) -> Result<Series> {
+                self.0.cast_with_dtype(data_type)
             }
 
             fn to_dummies(&self) -> Result<DataFrame> {
@@ -725,7 +686,7 @@ macro_rules! impl_dyn_series {
             }
 
             fn null_bits(&self) -> Vec<(usize, Option<Buffer>)> {
-                self.0.null_bits()
+                self.0.null_bits().collect()
             }
 
             fn reverse(&self) -> Series {
@@ -853,6 +814,11 @@ macro_rules! impl_dyn_series {
             fn peak_min(&self) -> BooleanChunked {
                 self.0.peak_min()
             }
+
+            #[cfg(feature = "is_in")]
+            fn is_in(&self, other: &Series) -> Result<BooleanChunked> {
+                IsIn::is_in(&self.0, other)
+            }
         }
     };
 }
