@@ -7,16 +7,15 @@ mod comparison;
 pub mod implementations;
 pub(crate) mod iterator;
 
-use crate::chunked_array::builder::get_list_builder;
-use crate::chunked_array::float::IsNan;
-use crate::series::arithmetic::coerce_lhs_rhs;
-use crate::utils::get_supertype;
+use crate::chunked_array::{builder::get_list_builder, float::IsNan, ChunkIdIter};
+use crate::utils::{split_ca, split_series};
+use crate::{series::arithmetic::coerce_lhs_rhs, POOL};
 use arrow::array::ArrayData;
 use arrow::compute::cast;
 use itertools::Itertools;
 use num::NumCast;
+use rayon::prelude::*;
 use std::any::Any;
-use std::borrow::Cow;
 use std::convert::TryFrom;
 use std::ops::Deref;
 use std::sync::Arc;
@@ -83,6 +82,10 @@ pub(crate) mod private {
         fn agg_median(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
             unimplemented!()
         }
+        #[cfg(feature = "lazy")]
+        fn agg_valid_count(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            unimplemented!()
+        }
         #[cfg(feature = "pivot")]
         fn pivot<'a>(
             &self,
@@ -142,8 +145,11 @@ pub(crate) mod private {
         fn zip_with_same_type(&self, _mask: &BooleanChunked, _other: &Series) -> Result<Series> {
             unimplemented!()
         }
-        fn is_in_same_type(&self, _list_array: &ListChunked) -> Result<BooleanChunked> {
-            unimplemented!()
+        #[cfg(feature = "sort_multiple")]
+        fn argsort_multiple(&self, _by: &[Series], _reverse: &[bool]) -> Result<UInt32Chunked> {
+            Err(PolarsError::InvalidOperation(
+                "argsort_multiple is not implemented for this Series".into(),
+            ))
         }
     }
 }
@@ -173,7 +179,7 @@ pub trait SeriesTrait: Send + Sync + private::PrivateSeries {
     }
 
     /// Get the lengths of the underlying chunks
-    fn chunk_lengths(&self) -> &Vec<usize> {
+    fn chunk_lengths(&self) -> ChunkIdIter {
         unimplemented!()
     }
     /// Name of series.
@@ -494,7 +500,7 @@ pub trait SeriesTrait: Send + Sync + private::PrivateSeries {
         unimplemented!()
     }
 
-    fn cast_with_datatype(&self, _data_type: &DataType) -> Result<Series> {
+    fn cast_with_dtype(&self, _data_type: &DataType) -> Result<Series> {
         unimplemented!()
     }
 
@@ -930,6 +936,13 @@ pub trait SeriesTrait: Send + Sync + private::PrivateSeries {
     fn peak_min(&self) -> BooleanChunked {
         unimplemented!()
     }
+
+    /// Check if elements of this Series are in the right Series, or List values of the right Series.
+    #[cfg(feature = "is_in")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "is_in")))]
+    fn is_in(&self, _other: &Series) -> Result<BooleanChunked> {
+        unimplemented!()
+    }
 }
 
 impl<'a> (dyn SeriesTrait + 'a) {
@@ -1086,7 +1099,7 @@ impl Series {
     where
         N: PolarsDataType,
     {
-        self.0.cast_with_datatype(&N::get_dtype())
+        self.0.cast_with_dtype(&N::get_dtype())
     }
     /// Returns `None` if the array is empty or only contains null values.
     /// ```
@@ -1220,20 +1233,6 @@ impl Series {
         lhs.zip_with_same_type(mask, rhs.as_ref())
     }
 
-    /// Check if values of this array are in the Series of the list array.
-    pub fn is_in(&self, list_array: &ListChunked) -> Result<BooleanChunked> {
-        let inner_dt = list_array.inner_dtype();
-        let my_dt = self.dtype();
-
-        let st = get_supertype(my_dt, &inner_dt)?;
-        let left = if &st != my_dt {
-            Cow::Owned(self.cast_with_datatype(&st)?)
-        } else {
-            Cow::Borrowed(self)
-        };
-        left.is_in_same_type(list_array)
-    }
-
     /// Cast a datelike Series to their physical representation.
     /// Primitives remain unchanged
     ///
@@ -1245,14 +1244,96 @@ impl Series {
     pub fn to_physical_repr(&self) -> Series {
         use DataType::*;
         let out = match self.dtype() {
-            Date32 => self.cast_with_datatype(&DataType::Int32),
-            Date64 => self.cast_with_datatype(&DataType::Int64),
-            Time64(_) => self.cast_with_datatype(&DataType::Int64),
-            Duration(_) => self.cast_with_datatype(&DataType::Int64),
+            Date32 => self.cast_with_dtype(&DataType::Int32),
+            Date64 => self.cast_with_dtype(&DataType::Int64),
+            Time64(_) => self.cast_with_dtype(&DataType::Int64),
+            Duration(_) => self.cast_with_dtype(&DataType::Int64),
             _ => return self.clone(),
         };
         out.unwrap()
     }
+
+    /// Take by index if ChunkedArray contains a single chunk.
+    ///
+    /// # Safety
+    /// This doesn't check any bounds. Null validity is checked.
+    pub unsafe fn take_unchecked_threaded(
+        &self,
+        idx: &UInt32Chunked,
+        rechunk: bool,
+    ) -> Result<Series> {
+        let n_threads = num_cpus::get();
+        let idx = split_ca(idx, n_threads)?;
+
+        let series: Result<Vec<_>> =
+            POOL.install(|| idx.par_iter().map(|idx| self.take_unchecked(idx)).collect());
+
+        let s = series?
+            .into_iter()
+            .reduce(|mut s, s1| {
+                s.append(&s1).unwrap();
+                s
+            })
+            .unwrap();
+        if rechunk {
+            Ok(s.rechunk())
+        } else {
+            Ok(s)
+        }
+    }
+
+    /// Take by index. This operation is clone.
+    ///
+    /// # Safety
+    ///
+    /// Out of bounds access doesn't Error but will return a Null value
+    pub fn take_threaded(&self, idx: &UInt32Chunked, rechunk: bool) -> Series {
+        let n_threads = num_cpus::get();
+        let idx = split_ca(idx, n_threads).unwrap();
+
+        let series: Vec<_> = POOL.install(|| idx.par_iter().map(|idx| self.take(idx)).collect());
+
+        let s = series
+            .into_iter()
+            .reduce(|mut s, s1| {
+                s.append(&s1).unwrap();
+                s
+            })
+            .unwrap();
+        if rechunk {
+            s.rechunk()
+        } else {
+            s
+        }
+    }
+
+    /// Filter by boolean mask. This operation clones data.
+    pub fn filter_threaded(&self, filter: &BooleanChunked, rechunk: bool) -> Result<Series> {
+        let n_threads = num_cpus::get();
+        let filters = split_ca(filter, n_threads).unwrap();
+        let series = split_series(self, n_threads).unwrap();
+
+        let series: Result<Vec<_>> = POOL.install(|| {
+            filters
+                .par_iter()
+                .zip(series)
+                .map(|(filter, s)| s.filter(filter))
+                .collect()
+        });
+
+        let s = series?
+            .into_iter()
+            .reduce(|mut s, s1| {
+                s.append(&s1).unwrap();
+                s
+            })
+            .unwrap();
+        if rechunk {
+            Ok(s.rechunk())
+        } else {
+            Ok(s)
+        }
+    }
 }
 
 impl Deref for Series {
